@@ -1,20 +1,60 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/savage/savage_driver.h,v 1.16 2003/01/18 15:22:30 eich Exp $ */
+/*
+ * Copyright (C) 1994-2000 The XFree86 Project, Inc.  All Rights Reserved.
+ * Copyright (c) 2003-2006, X.Org Foundation
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
+ * COPYRIGHT HOLDER(S) OR AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
+ *
+ * Except as contained in this notice, the name of the copyright holder(s)
+ * and author(s) shall not be used in advertising or otherwise to promote
+ * the sale, use or other dealings in this Software without prior written
+ * authorization from the copyright holder(s) and author(s).
+ */
 
-#ifndef SAVAGE_VGAHWMMIO_H
-#define SAVAGE_VGAHWMMIO_H
+#ifndef SAVAGE_DRIVER_H
+#define SAVAGE_DRIVER_H
 
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
 
+#ifdef XSERVER_LIBPCIACCESS
+#include <pciaccess.h>
+#define VENDOR_ID(p)      (p)->vendor_id
+#define DEVICE_ID(p)      (p)->device_id
+#define SUBSYS_ID(p)      (p)->subdevice_id
+#define CHIP_REVISION(p)  (p)->revision
+#else
+#define VENDOR_ID(p)      (p)->vendor
+#define DEVICE_ID(p)      (p)->chipType
+#define SUBSYS_ID(p)      (p)->subsysCard
+#define CHIP_REVISION(p)  (p)->chipRev
+#endif
+
 #define MODE_24 24
 
 #include "compiler.h"
 #include "vgaHW.h"
 #include "xf86.h"
+#if GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) < 6
 #include "xf86Resources.h"
+#endif
 #include "xf86Pci.h"
-#include "xf86PciInfo.h"
 #include "xf86_OSproc.h"
 #include "xf86Cursor.h"
 #include "mipointer.h"
@@ -24,12 +64,17 @@
 #include "xf86cmap.h"
 #include "vbe.h"
 #include "xaa.h"
+#include "exa.h"
 #include "xf86xv.h"
 
 #include "savage_regs.h"
 #include "savage_vbe.h"
 
-#ifdef XF86DRI
+#ifndef XF86DRI
+#undef SAVAGEDRI
+#endif
+
+#ifdef SAVAGEDRI
 #define _XF86DRI_SERVER_
 #include "savage_dripriv.h"
 #include "savage_dri.h"
@@ -37,6 +82,8 @@
 #include "GL/glxint.h"
 #include "xf86drm.h"
 
+#include "compat-api.h"
+
 /* Totals 2 Mbytes which equals 2^16 32-byte vertices divided among up
  * to 32 clients. */
 #define SAVAGE_NUM_BUFFERS 32
@@ -96,6 +143,9 @@ typedef struct _server{
 
    /* command DMA */
    drmRegion cmdDma;
+
+   /* XVideo through AGP */
+   drmRegion agpXVideo;
 } SAVAGEDRIServerPrivateRec, *SAVAGEDRIServerPrivatePtr;
 
 #endif
@@ -151,10 +201,10 @@ typedef struct
 #define SAVAGE_TV_ON	4
 
 #define SAVAGE_DRIVER_NAME	"savage"
-#define SAVAGE_DRIVER_VERSION	"2.1.0"
-#define SAVAGE_VERSION_MAJOR	2
-#define SAVAGE_VERSION_MINOR	1
-#define SAVAGE_PATCHLEVEL	0
+#define SAVAGE_DRIVER_VERSION	PACKAGE_VERSION
+#define SAVAGE_VERSION_MAJOR	PACKAGE_VERSION_MAJOR
+#define SAVAGE_VERSION_MINOR	PACKAGE_VERSION_MINOR
+#define SAVAGE_PATCHLEVEL	PACKAGE_VERSION_PATCHLEVEL
 #define SAVAGE_VERSION	((SAVAGE_VERSION_MAJOR << 24) | \
 			 (SAVAGE_VERSION_MINOR << 16) | \
 			 SAVAGE_PATCHLEVEL)
@@ -239,6 +289,17 @@ typedef struct _StatInfo {
     int     realOff;
 } StatInfoRec,*StatInfoPtr;
 
+struct savage_region {
+#ifdef XSERVER_LIBPCIACCESS
+    pciaddr_t       base;
+    pciaddr_t       size;
+#else
+    unsigned long   base;
+    unsigned long   size;
+#endif
+    void          * memory;
+};
+
 typedef struct _Savage {
     SavageRegRec	SavedReg;
     SavageRegRec	ModeReg;
@@ -260,15 +321,15 @@ typedef struct _Savage {
     int			endfb;
 
     /* These are physical addresses. */
-    unsigned long	FrameBufferBase;
-    unsigned long	MmioBase;
-    unsigned long	ApertureBase;
     unsigned long	ShadowPhysical;
 
     /* These are linear addresses. */
+    struct savage_region   MmioRegion;
+    struct savage_region   FbRegion;
+    struct savage_region   ApertureRegion;
+
     unsigned char*	MapBase;
     unsigned char*	BciMem;
-    unsigned char*	MapBaseDense;
     unsigned char*	FBBase;
     unsigned char*	ApertureMap;
     unsigned char*	FBStart;
@@ -284,6 +345,7 @@ typedef struct _Savage {
     /* Here are all the Options */
 
     OptionInfoPtr	Options;
+    Bool		IgnoreEDID;
     Bool		ShowCache;
     Bool		pci_burst;
     Bool		NoPCIRetry;
@@ -325,8 +387,12 @@ typedef struct _Savage {
     int			TVSizeY;
 
     CloseScreenProcPtr	CloseScreen;
+#ifdef XSERVER_LIBPCIACCESS
+    struct pci_device * PciInfo;
+#else
     pciVideoPtr		PciInfo;
     PCITAG		PciTag;
+#endif
     int			Chipset;
     int			ChipId;
     int			ChipRev;
@@ -343,7 +409,16 @@ typedef struct _Savage {
     /* Support for shadowFB and rotation */
     unsigned char *	ShadowPtr;
     int			ShadowPitch;
-    void		(*PointerMoved)(int index, int x, int y);
+    void		(*PointerMoved)(SCRN_ARG_TYPE arg, int x, int y);
+
+    /* support for EXA */
+    ExaDriverPtr        EXADriverPtr;
+    Bool		useEXA;
+    unsigned long	EXAendfb;
+    unsigned long 	pbd_offset;
+    unsigned long	sbd_offset;
+    unsigned long	pbd_high;
+    unsigned long	sbd_high;
 
     /* Support for XAA acceleration */
     XAAInfoRecPtr	AccelInfoRec;
@@ -388,7 +463,7 @@ typedef struct _Savage {
      int                 overlayDepth;
      int			primStreamBpp;
 
-#ifdef XF86DRI
+#ifdef SAVAGEDRI
     int 		LockHeld;
     Bool 		directRenderingEnabled;
     DRIInfoPtr 		pDRIInfo;
@@ -397,7 +472,8 @@ typedef struct _Savage {
     __GLXvisualConfig*	pVisualConfigs;
     SAVAGEConfigPrivPtr 	pVisualConfigsPriv;
     SAVAGEDRIServerPrivatePtr DRIServerInfo;
-
+    ScreenWakeupHandlerProcPtr coreWakeupHandler;
+    ScreenBlockHandlerProcPtr  coreBlockHandler;
 
 #if 0
     Bool		haveQuiescense;
@@ -420,6 +496,7 @@ typedef struct _Savage {
 
     Bool bDisableXvMC;
 
+    Bool AGPforXv;
 #endif
 
     Bool disableCOB;
@@ -476,7 +553,7 @@ typedef struct _Savage {
 #define writescan savagewritescan
 
 /* add for support DRI */
-#ifdef XF86DRI
+#ifdef SAVAGEDRI
 
 #define SAVAGE_FRONT	0x1
 #define SAVAGE_BACK	0x2
@@ -487,20 +564,7 @@ Bool SAVAGEDRIScreenInit( ScreenPtr pScreen );
 Bool SAVAGEInitMC(ScreenPtr pScreen);
 void SAVAGEDRICloseScreen( ScreenPtr pScreen );
 Bool SAVAGEDRIFinishScreenInit( ScreenPtr pScreen );
-
-Bool SAVAGELockUpdate( ScrnInfoPtr pScrn, drmLockFlags flags );
-
-#if 0
-void SAVAGEGetQuiescence( ScrnInfoPtr pScrn );
-void SAVAGEGetQuiescenceShared( ScrnInfoPtr pScrn );
-#endif
-
-void SAVAGESelectBuffer(ScrnInfoPtr pScrn, int which);
-
-#if 0
-Bool SAVAGECleanupDma(ScrnInfoPtr pScrn);
-Bool SAVAGEInitDma(ScrnInfoPtr pScrn, int prim_size);
-#endif
+void SAVAGEDRIResume( ScreenPtr pScreen );
 
 #define SAVAGE_AGP_1X_MODE		0x01
 #define SAVAGE_AGP_2X_MODE		0x02
@@ -516,9 +580,9 @@ extern void SavageCommonCalcClock(long freq, int min_m, int min_n1,
 			int max_n1, int min_n2, int max_n2,
 			long freq_min, long freq_max,
 			unsigned char *mdiv, unsigned char *ndiv);
-void SavageAdjustFrame(int scrnIndex, int y, int x, int flags);
+void SavageAdjustFrame(ADJUST_FRAME_ARGS_DECL);
 void SavageDoAdjustFrame(ScrnInfoPtr pScrn, int y, int x, int crtc2);
-Bool SavageSwitchMode(int scrnIndex, DisplayModePtr mode, int flags);
+Bool SavageSwitchMode(SWITCH_MODE_ARGS_DECL);
 
 /* In savage_cursor.c. */
 
@@ -534,13 +598,17 @@ void SavageSetGBD(ScrnInfoPtr);
 void SavageAccelSync(ScrnInfoPtr);
 /*int SavageHelpSolidROP(ScrnInfoPtr pScrn, int *fg, int pm, int *rop);*/
 
+/* XAA and EXA */
+Bool SavageXAAInit(ScreenPtr);
+Bool SavageEXAInit(ScreenPtr);
+
 /* In savage_i2c.c. */
 
 Bool SavageI2CInit(ScrnInfoPtr pScrn);
 
 /* In savage_shadow.c */
 
-void SavagePointerMoved(int index, int x, int y);
+void SavagePointerMoved(SCRN_ARG_TYPE arg, int x, int y);
 void SavageRefreshArea(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
 void SavageRefreshArea8(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
 void SavageRefreshArea16(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
@@ -559,6 +627,7 @@ ModeStatus SavageMatchBiosMode(ScrnInfoPtr pScrn,int width,int height,int refres
 
 unsigned short SavageGetBIOSModes( 
     SavagePtr psav,
+    VbeInfoBlock *vbe,
     int iDepth,
     SavageModeEntryPtr s3vModeTable );
 
@@ -587,5 +656,5 @@ void SavageInitStreams2000(ScrnInfoPtr pScrn);
 #define DEPTH_2ND(pScrn) (pScrn->depth > 8 ? pScrn->depth\
                               : SAVPTR(pScrn)->overlayDepth)
 
-#endif /* SAVAGE_VGAHWMMIO_H */
+#endif /* SAVAGE_DRIVER_H */
 
